#ifndef SPARSE_MATRIX
#define SPARSE_MATRIX

#include <ext/hash_map>

/* in order to save some space we use a hash_map for storing the entries of the sparse
   matrices that allow fast acces to the basepairs (i,j) in a structure.
*/

namespace __gnu_cxx {
    //! defines a new hash function for pairs of ints
    template<>
    struct hash<std::pair<size_t,size_t> >
    {
	size_t
	operator()(std::pair<size_t,size_t> p) const
	{ return p.first<<(sizeof(size_t)/2) | p.second; }
    };
}

//! represents a sparse 2D matrix of entries val_t
//! implements the matrix by a hash map
template <class T>
class SparseMatrix {
public:
  typedef T val_t;
  typedef std::pair<size_t,size_t> key_t;
protected:
    typedef __gnu_cxx::hash_map<key_t,val_t > map_t;  
    map_t the_map;
    val_t def;
public:

    typedef typename map_t::const_iterator const_iterator;

    SparseMatrix(val_t deflt) : the_map(),def(deflt) {}
  
    const val_t & operator() (size_t i, size_t j) const {
	const_iterator it = the_map.find(key_t(i,j));
	if ( it == the_map.end() ) 
	    return def;
	else 
	    return it->second;
    }
  
  
    void set(size_t i, size_t j, const val_t &val) {
	the_map[std::pair<int,int>(i,j)]=val;
    }
  
  
    const_iterator begin() const {
	return the_map.begin();
    }
    const_iterator end() const {
	return the_map.end();
    }
  
};

#endif // SPARSE_MATRIX
